@@ -25,9 +25,8 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-#pragma once
-
-
+#ifndef _RDKAFKA_ASSIGNOR_H_
+#define _RDKAFKA_ASSIGNOR_H_
 
 
 
@@ -36,6 +35,7 @@ typedef struct rd_kafka_group_member_s {
         rd_kafka_topic_partition_list_t *rkgm_assignment;
         rd_list_t                        rkgm_eligible;
         rd_kafkap_str_t                 *rkgm_member_id;
+        rd_kafkap_str_t                 *rkgm_group_instance_id;
         rd_kafkap_bytes_t               *rkgm_userdata;
         rd_kafkap_bytes_t               *rkgm_member_metadata;
 } rd_kafka_group_member_t;
@@ -157,3 +157,4 @@ rd_kafka_roundrobin_assignor_assign_cb (rd_kafka_t *rk,
 					char *errstr, size_t errstr_size,
 					void *opaque);
 
+#endif /* _RDKAFKA_ASSIGNOR_H_ */
