@@ -828,8 +828,8 @@ rd_kafka_msgset_reader_msg_v2 (rd_kafka_msgset_reader_t *msetr) {
                                 rd_rkb_log(msetr->msetr_rkb, LOG_ERR, "TXN",
                                         "%s [%"PRId32"]: "
                                         "Abort txn ctrl msg bad order "
-                                        "at offset %"PRId64". Expected "
-                                        "before or at %"PRId64". Messages "
+                                        "at offset %"PRId64": expected "
+                                        "before or at %"PRId64": messages "
                                         "in aborted transactions may be "
                                         "delivered to the application",
                                         rktp->rktp_rkt->rkt_topic->str,
@@ -861,6 +861,12 @@ unexpected_abort_txn:
                         break;
                 }
 
+                rko = rd_kafka_op_new_ctrl_msg(
+                        rktp, msetr->msetr_tver->version,
+                        rkbuf, hdr.Offset);
+                rd_kafka_q_enq(&msetr->msetr_rkq, rko);
+                msetr->msetr_msgcnt++;
+
                 return RD_KAFKA_RESP_ERR_NO_ERROR;
         }
 
@@ -1073,17 +1079,6 @@ rd_kafka_msgset_reader_v2 (rd_kafka_msgset_reader_t *msetr) {
                                               &save_slice, payload_size))
                         rd_kafka_buf_check_len(rkbuf, payload_size);
 
-                if (msetr->msetr_aborted_txns == NULL &&
-                    msetr->msetr_v2_hdr->Attributes &
-                    RD_KAFKA_MSGSET_V2_ATTR_CONTROL) {
-                        /* Since there are no aborted transactions,
-                         * the MessageSet must correspond to a commit marker.
-                         * These are ignored. */
-                        rd_kafka_buf_skip(rkbuf, payload_size);
-                        rd_slice_widen(&rkbuf->rkbuf_reader, &save_slice);
-                        goto done;
-                }
-
                 if (msetr->msetr_aborted_txns != NULL &&
                     (msetr->msetr_v2_hdr->Attributes &
                      (RD_KAFKA_MSGSET_V2_ATTR_TRANSACTIONAL|
@@ -1103,10 +1098,12 @@ rd_kafka_msgset_reader_v2 (rd_kafka_msgset_reader_t *msetr) {
                                 rd_rkb_dbg(msetr->msetr_rkb, MSG, "MSG",
                                            "%s [%"PRId32"]: "
                                            "Skipping %"PRId32" message(s) "
-                                           "in aborted transaction",
+                                           "in aborted transaction "
+                                           "at offset %"PRId64,
                                            rktp->rktp_rkt->rkt_topic->str,
                                            rktp->rktp_partition,
-                                           msetr->msetr_v2_hdr->RecordCount);
+                                           msetr->msetr_v2_hdr->RecordCount,
+                                           txn_start_offset);
                                 rd_kafka_buf_skip(rkbuf, payload_size);
                                 rd_slice_widen(&rkbuf->rkbuf_reader,
                                                &save_slice);
