@@ -67,7 +67,8 @@ typedef struct rd_kafka_replyq_s {
 #define RD_KAFKA_OP_F_BLOCKING    0x8  /* rkbuf: blocking protocol request */
 #define RD_KAFKA_OP_F_REPROCESS   0x10 /* cgrp: Reprocess at a later time. */
 #define RD_KAFKA_OP_F_SENT        0x20 /* rkbuf: request sent on wire */
-
+#define RD_KAFKA_OP_F_FLEXVER     0x40 /* rkbuf: flexible protocol version
+                                        *        (KIP-482) */
 
 typedef enum {
         RD_KAFKA_OP_NONE,     /* No specific type, use OP_CB */
@@ -121,6 +122,8 @@ typedef enum {
         RD_KAFKA_OP_CONNECT,         /**< Connect (to broker) */
         RD_KAFKA_OP_OAUTHBEARER_REFRESH, /**< Refresh OAUTHBEARER token */
         RD_KAFKA_OP_MOCK,            /**< Mock cluster command */
+        RD_KAFKA_OP_BROKER_MONITOR,    /**< Broker state change */
+        RD_KAFKA_OP_TXN,             /**< Transaction command */
         RD_KAFKA_OP__END
 } rd_kafka_op_type_t;
 
@@ -427,34 +430,68 @@ struct rd_kafka_op_s {
                 struct {
                         enum {
                                 RD_KAFKA_MOCK_CMD_TOPIC_SET_ERROR,
+                                RD_KAFKA_MOCK_CMD_TOPIC_CREATE,
                                 RD_KAFKA_MOCK_CMD_PART_SET_LEADER,
                                 RD_KAFKA_MOCK_CMD_PART_SET_FOLLOWER,
                                 RD_KAFKA_MOCK_CMD_PART_SET_FOLLOWER_WMARKS,
-                                RD_KAFKA_MOCK_CMD_BROKER_SET_RACK
+                                RD_KAFKA_MOCK_CMD_BROKER_SET_UPDOWN,
+                                RD_KAFKA_MOCK_CMD_BROKER_SET_RACK,
+                                RD_KAFKA_MOCK_CMD_COORD_SET,
+                                RD_KAFKA_MOCK_CMD_APIVERSION_SET,
                         } cmd;
 
                         rd_kafka_resp_err_t err; /**< Error for:
                                                   *    TOPIC_SET_ERROR */
                         char *name;              /**< For:
                                                   *    TOPIC_SET_ERROR
+                                                  *    TOPIC_CREATE
                                                   *    PART_SET_FOLLOWER
                                                   *    PART_SET_FOLLOWER_WMARKS
-                                                  *    BROKER_SET_RACK */
+                                                  *    BROKER_SET_RACK
+                                                  *    COORD_SET (key_type) */
+                        char *str;               /**< For:
+                                                  *    COORD_SET (key) */
                         int32_t partition;       /**< For:
                                                   *    PART_SET_FOLLOWER
                                                   *    PART_SET_FOLLOWER_WMARKS
-                                                  *    PART_SET_LEADER */
+                                                  *    PART_SET_LEADER
+                                                  *    APIVERSION_SET (ApiKey)
+                                                  */
                         int32_t broker_id;       /**< For:
                                                   *    PART_SET_FOLLOWER
                                                   *    PART_SET_LEADER
-                                                  *    BROKER_SET_RACK */
+                                                  *    BROKER_SET_UPDOWN
+                                                  *    BROKER_SET_RACK
+                                                  *    COORD_SET */
                         int64_t lo;              /**< Low offset, for:
+                                                  *    TOPIC_CREATE (part cnt)
                                                   *    PART_SET_FOLLOWER_WMARKS
+                                                  *    BROKER_SET_UPDOWN
+                                                  *    APIVERSION_SET (minver)
                                                   */
                         int64_t hi;              /**< High offset, for:
+                                                  *    TOPIC_CREATE (repl fact)
                                                   *    PART_SET_FOLLOWER_WMARKS
+                                                  *    APIVERSION_SET (maxver)
                                                   */
                 } mock;
+
+                struct {
+                        struct rd_kafka_broker_s *rkb; /**< Broker who's state
+                                                        *   changed. */
+                        /**< Callback to trigger on the op handler's thread. */
+                        void (*cb) (struct rd_kafka_broker_s *rkb);
+                } broker_monitor;
+
+                struct {
+                        rd_kafka_error_t *error; /**< Error object */
+                        char *group_id; /**< Consumer group id for commits */
+                        int   timeout_ms; /**< Operation timeout */
+                        rd_ts_t abs_timeout; /**< Absolute time */
+                        /**< Offsets to commit */
+                        rd_kafka_topic_partition_list_t *offsets;
+                } txn;
+
         } rko_u;
 };
 
@@ -483,7 +520,6 @@ int rd_kafka_op_reply (rd_kafka_op_t *rko, rd_kafka_resp_err_t err);
 
 #define rd_kafka_op_set_prio(rko,prio) ((rko)->rko_prio = prio)
 
-
 #define rd_kafka_op_err(rk,err,...) do {				\
 		if (!((rk)->rk_conf.enabled_events & RD_KAFKA_EVENT_ERROR)) { \
 			rd_kafka_log(rk, LOG_ERR, "ERROR", __VA_ARGS__); \
@@ -497,6 +533,10 @@ void rd_kafka_q_op_err (rd_kafka_q_t *rkq, rd_kafka_op_type_t optype,
                         rd_kafka_resp_err_t err, int32_t version,
                         rd_kafka_toppar_t *rktp, int64_t offset,
 			const char *fmt, ...);
+rd_kafka_op_t *rd_kafka_op_req0 (rd_kafka_q_t *destq,
+                                 rd_kafka_q_t *recvq,
+                                 rd_kafka_op_t *rko,
+                                 int timeout_ms);
 rd_kafka_op_t *rd_kafka_op_req (rd_kafka_q_t *destq,
                                 rd_kafka_op_t *rko,
                                 int timeout_ms);
@@ -516,6 +556,12 @@ rd_kafka_op_new_fetch_msg (rd_kafka_msg_t **rkmp,
                            size_t key_len, const void *key,
                            size_t val_len, const void *val);
 
+rd_kafka_op_t *
+rd_kafka_op_new_ctrl_msg (rd_kafka_toppar_t *rktp,
+                           int32_t version,
+                           rd_kafka_buf_t *rkbuf,
+                           int64_t offset);
+
 void rd_kafka_op_throttle_time (struct rd_kafka_broker_s *rkb,
 				rd_kafka_q_t *rkq,
 				int throttle_time);
@@ -531,7 +577,12 @@ extern rd_atomic32_t rd_kafka_op_cnt;
 
 void rd_kafka_op_print (FILE *fp, const char *prefix, rd_kafka_op_t *rko);
 
-void rd_kafka_op_offset_store (rd_kafka_t *rk, rd_kafka_op_t *rko,
-			       const rd_kafka_message_t *rkmessage);
+void rd_kafka_op_offset_store (rd_kafka_t *rk, rd_kafka_op_t *rko);
+
+
+#define rd_kafka_op_is_ctrl_msg(rko)                                    \
+        ((rko)->rko_type == RD_KAFKA_OP_FETCH &&                        \
+         !(rko)->rko_err &&                                             \
+         ((rko)->rko_u.fetch.rkm.rkm_flags & RD_KAFKA_MSG_F_CONTROL))
 
 #endif /* _RDKAFKA_OP_H_ */
