@@ -44,6 +44,9 @@ typedef struct rd_kafka_s rd_kafka_t;
 #define _C_CYA "\033[36m"
 
 
+/** Test scenario */
+extern char test_scenario[64];
+
 /** @returns the \p msecs timeout multiplied by the test timeout multiplier */
 extern int tmout_multip (int msecs);
 
@@ -57,6 +60,9 @@ extern int test_quick;
 #define TEST_BRKVER_X(V,I) \
         (((V) >> (24-((I)*8))) & 0xff)
 
+/** @brief Topic Admin API supported by this broker version and later */
+#define TEST_BRKVER_TOPIC_ADMINAPI TEST_BRKVER(0,10,2,0)
+
 extern int test_broker_version;
 extern int test_on_ci;
 
@@ -73,7 +79,71 @@ test_produce_msgs_easy_size (const char *topic, uint64_t testid,
 #define test_produce_msgs_easy(topic,testid,partition,msgcnt) \
         test_produce_msgs_easy_size(topic,testid,partition,msgcnt,0)
 
-void test_FAIL (const char *file, int line, int fail_now, const char *str);
+
+void test_fail0 (const char *file, int line, const char *function,
+                 int do_lock, int fail_now, const char *fmt, ...)
+        RD_FORMAT(printf, 6, 7);
+
+
+
+void test_fail0 (const char *file, int line, const char *function,
+                 int do_lock, int fail_now, const char *fmt, ...)
+        RD_FORMAT(printf, 6, 7);
+
+#define TEST_FAIL0(file,line,do_lock,fail_now,...)   \
+        test_fail0(__FILE__, __LINE__, __FUNCTION__, \
+                   do_lock, fail_now, __VA_ARGS__)
+
+/* Whine and abort test */
+#define TEST_FAIL(...) TEST_FAIL0(__FILE__,__LINE__,1,1,__VA_ARGS__)
+
+/* Whine right away, mark the test as failed, but continue the test. */
+#define TEST_FAIL_LATER(...) TEST_FAIL0(__FILE__,__LINE__,1,0,__VA_ARGS__)
+
+/* Whine right away, maybe mark the test as failed, but continue the test. */
+#define TEST_FAIL_LATER0(LATER,...) TEST_FAIL0(__FILE__,__LINE__,1,!(LATER),__VA_ARGS__)
+
+#define TEST_FAILCNT()  (test_curr->failcnt)
+
+#define TEST_LATER_CHECK(...) do {                              \
+        if (test_curr->state == TEST_FAILED)                    \
+                TEST_FAIL("See previous errors. " __VA_ARGS__); \
+        } while (0)
+
+#define TEST_PERROR(call) do {                                         \
+               if (!(call))                                            \
+                       TEST_FAIL(#call " failed: %s", rd_strerror(errno)); \
+       } while (0)
+
+#define TEST_WARN(...) do {                                              \
+                fprintf(stderr, "\033[33m[%-28s/%7.3fs] WARN: ",       \
+                       test_curr->name,                                \
+                       test_curr->start ?                              \
+                       ((float)(test_clock() -                         \
+                                 test_curr->start)/1000000.0f) : 0);    \
+               fprintf(stderr, __VA_ARGS__);                           \
+                fprintf(stderr, "\033[0m");                             \
+       } while (0)
+
+/* "..." is a failure reason in printf format, include as much info as needed */
+#define TEST_ASSERT(expr,...) do {            \
+        if (!(expr)) {                        \
+                      TEST_FAIL("Test assertion failed: \"" # expr  "\": " \
+                                __VA_ARGS__);                           \
+                      }                                                 \
+        } while (0)
+
+
+/* "..." is a failure reason in printf format, include as much info as needed */
+#define TEST_ASSERT_LATER(expr,...) do {                                \
+                if (!(expr)) {                                          \
+                        TEST_FAIL0(__FILE__, __LINE__, 1, 0,            \
+                                   "Test assertion failed: \"" # expr  "\": " \
+                                   __VA_ARGS__);                        \
+                }                                                       \
+        } while (0)
+
+
 void test_SAY (const char *file, int line, int level, const char *str);
 void test_SKIP (const char *file, int line, const char *str);
 
@@ -155,7 +225,7 @@ static RD_INLINE int64_t test_clock (void) {
 
 
 typedef struct test_timing_s {
-        char name[64];
+        char name[256];
         int64_t ts_start;
         int64_t duration;
         int64_t ts_every; /* Last every */
@@ -189,7 +259,7 @@ typedef struct test_timing_s {
 
 #else
 #define TIMING_STOP(TIMING) do {                                        \
-        char _str[128];                                                 \
+        char _str[512];                                                 \
         (TIMING)->duration = test_clock() - (TIMING)->ts_start;         \
         rd_snprintf(_str, sizeof(_str), "%s: duration %.3fms\n",        \
                     (TIMING)->name, (float)(TIMING)->duration / 1000.0f); \
